/*********************************************************************
 *                                                                   *
 *                     XGS : Apple IIGS Emulator                     *
 *                                                                   *
 *        Written and Copyright (C)1996 by Joshua M. Thompson        *
 *                                                                   *
 *  You are free to distribute this code for non-commercial purposes *
 * I ask only that you notify me of any changes you make to the code *
 *     Commercial use is prohibited without my written permission    *
 *                                                                   *
 *********************************************************************/

/*
 * File: fault.c
 *
 * Contains generic read/write fault handling routines for special
 * memory banks. This includes reads and write to the page $C0 I/O
 * space.
 */

#include <xgs.h>
#include "memory.h"
#include "../adb/adb.h"
#include "../clock/clock.h"
#include "../sound/sound.h"
#include "../video/video.h"

int diskfake;

byte diskfake1(byte val)
{
	return 0x80;
}

byte diskfake2(byte val)
{
	return diskfake;
}

byte diskfake3(byte val)
{
	diskfake = (val | 0x80) & ~0x20;
	return diskfake;
}

byte (*mem_io_read_dispatch[256])(byte) = {
	ADB_readKey,		/* $C000 */
	MEM_set80store,		/* $C001 */
	MEM_clearAuxRd,		/* $C002 */
	MEM_setAuxRd,		/* $C003 */
	MEM_clearAuxWrt,	/* $C004 */
	MEM_setAuxWrt,		/* $C005 */
	NULL,			/* $C006 */
	NULL,			/* $C007 */
	MEM_clearAltZP,		/* $C008 */
	MEM_setAltZP,		/* $C009 */
	NULL,			/* $C00A */
	NULL,			/* $C00B */
	VID_clear80col,		/* $C00C */
	VID_set80col,		/* $C00D */
	VID_clearAltCh,		/* $C00E */
	VID_setAltCh,		/* $C00F */
	ADB_clearKey,		/* $C010 */
	MEM_getLCbank,		/* $C011 */
	MEM_getLCread,		/* $C012 */
	MEM_getAuxRd,		/* $C013 */
	MEM_getAuxWrt,		/* $C014 */
	NULL,			/* $C015 */
	MEM_getAltZP,		/* $C016 */
	NULL,			/* $C017 */
	MEM_get80store,		/* $C018 */
	VID_getVBL,		/* $C019 */
	VID_getText,		/* $C01A */
	VID_getMixed,		/* $C01B */
	VID_getPage2,		/* $C01C */
	VID_getHires,		/* $C01D */
	VID_getAltCh,		/* $C01E */
	VID_get80col,		/* $C01F */
	NULL,			/* $C020 */
	NULL,			/* $C021 */
	VID_getColorReg,	/* $C022 */
	NULL,			/* $C023 */
	ADB_readMouse,		/* $C024 */
	ADB_readModifiers,	/* $C025 */
	ADB_readCommand,	/* $C026 */
	ADB_readStatus,		/* $C027 */
	NULL,			/* $C028 */
	VID_getNewVideo,	/* $C029 */
	NULL,			/* $C02A */
	NULL,			/* $C02B */
	NULL,			/* $C02C */
	MEM_getSlotReg,		/* $C02D */
	NULL,			/* $C02E */
	NULL,			/* $C02F */
	NULL,			/* $C030 */
	NULL,			/* $C031 */
	NULL,			/* $C032 */
	CLK_getData,		/* $C033 */
	CLK_getControl,		/* $C034 */
	MEM_getShadowReg,	/* $C035 */
	NULL,			/* $C036 */
	NULL,			/* $C037 */
	NULL,			/* $C038 */
	NULL,			/* $C039 */
	NULL,			/* $C03A */
	NULL,			/* $C03B */
	SND_readSoundCtl,	/* $C03C */
	SND_readSoundData,	/* $C03D */
	SND_readSoundAddrL,	/* $C03E */
	SND_readSoundAddrH,	/* $C03F */
	NULL,			/* $C040 */
	MEM_getIntEnReg,	/* $C041 */
	NULL,			/* $C042 */
	NULL,			/* $C043 */
	NULL,			/* $C044 */
	NULL,			/* $C045 */
	MEM_getDiagType,	/* $C046 */
	NULL,			/* $C047 */
	NULL,			/* $C048 */
	NULL,			/* $C049 */
	NULL,			/* $C04A */
	NULL,			/* $C04B */
	NULL,			/* $C04C */
	NULL,			/* $C04D */
	NULL,			/* $C04E */
	NULL,			/* $C04F */
	VID_clearText,		/* $C050 */
	VID_setText,		/* $C051 */
	VID_clearMixed,		/* $C052 */
	VID_setMixed,		/* $C053 */
	VID_clearPage2,		/* $C054 */
	VID_setPage2,		/* $C055 */
	VID_clearHires,		/* $C056 */
	VID_setHires,		/* $C057 */
	NULL,			/* $C058 */
	NULL,			/* $C059 */
	NULL,			/* $C05A */
	NULL,			/* $C05B */
	NULL,			/* $C05C */
	NULL,			/* $C05D */
	VID_setDblRes,		/* $C05E */
	VID_clearDblRes,	/* $C05F */
	NULL,			/* $C060 */
	ADB_readCommandKey,	/* $C061 */
	ADB_readOptionKey,	/* $C062 */
	NULL,			/* $C063 */
	NULL,			/* $C064 */
	NULL,			/* $C065 */
	NULL,			/* $C066 */
	NULL,			/* $C067 */
	MEM_getStateReg,	/* $C068 */
	NULL,			/* $C069 */
	NULL,			/* $C06A */
	NULL,			/* $C06B */
	NULL,			/* $C06C */
	NULL,			/* $C06D */
	NULL,			/* $C06E */
	NULL,			/* $C06F */
	NULL,			/* $C070 */
	MEM_getC071,		/* $C071 */
	MEM_getC072,		/* $C072 */
	MEM_getC073,		/* $C073 */
	MEM_getC074,		/* $C074 */
	MEM_getC075,		/* $C075 */
	MEM_getC076,		/* $C076 */
	MEM_getC077,		/* $C077 */
	MEM_getC078,		/* $C078 */
	MEM_getC079,		/* $C079 */
	MEM_getC07A,		/* $C07A */
	MEM_getC07B,		/* $C07B */
	MEM_getC07C,		/* $C07C */
	MEM_getC07D,		/* $C07D */
	MEM_getC07E,		/* $C07E */
	MEM_getC07F,		/* $C07F */
	MEM_setLCx80,		/* $C080 */
	MEM_setLCx81,		/* $C081 */
	MEM_setLCx82,		/* $C082 */
	MEM_setLCx83,		/* $C083 */
	NULL,			/* $C084 */
	NULL,			/* $C085 */
	NULL,			/* $C086 */
	NULL,			/* $C087 */
	MEM_setLCx88,		/* $C088 */
	MEM_setLCx89,		/* $C089 */
	MEM_setLCx8A,		/* $C08A */
	MEM_setLCx8B,		/* $C08B */
	NULL,			/* $C08C */
	NULL,			/* $C08D */
	NULL,			/* $C08E */
	NULL,			/* $C08F */
	NULL,			/* $C090 */
	NULL,			/* $C091 */
	NULL,			/* $C092 */
	NULL,			/* $C093 */
	NULL,			/* $C094 */
	NULL,			/* $C095 */
	NULL,			/* $C096 */
	NULL,			/* $C097 */
	NULL,			/* $C098 */
	NULL,			/* $C099 */
	NULL,			/* $C09A */
	NULL,			/* $C09B */
	NULL,			/* $C09C */
	NULL,			/* $C09D */
	NULL,			/* $C09E */
	NULL,			/* $C09F */
	NULL,			/* $C0A0 */
	NULL,			/* $C0A1 */
	NULL,			/* $C0A2 */
	NULL,			/* $C0A3 */
	NULL,			/* $C0A4 */
	NULL,			/* $C0A5 */
	NULL,			/* $C0A6 */
	NULL,			/* $C0A7 */
	NULL,			/* $C0A8 */
	NULL,			/* $C0A9 */
	NULL,			/* $C0AA */
	NULL,			/* $C0AB */
	NULL,			/* $C0AC */
	NULL,			/* $C0AD */
	NULL,			/* $C0AE */
	NULL,			/* $C0AF */
	NULL,			/* $C0B0 */
	NULL,			/* $C0B1 */
	NULL,			/* $C0B2 */
	NULL,			/* $C0B3 */
	NULL,			/* $C0B4 */
	NULL,			/* $C0B5 */
	NULL,			/* $C0B6 */
	NULL,			/* $C0B7 */
	NULL,			/* $C0B8 */
	NULL,			/* $C0B9 */
	NULL,			/* $C0BA */
	NULL,			/* $C0BB */
	NULL,			/* $C0BC */
	NULL,			/* $C0BD */
	NULL,			/* $C0BE */
	NULL,			/* $C0BF */
	NULL,			/* $C0C0 */
	NULL,			/* $C0C1 */
	NULL,			/* $C0C2 */
	NULL,			/* $C0C3 */
	NULL,			/* $C0C4 */
	NULL,			/* $C0C5 */
	NULL,			/* $C0C6 */
	NULL,			/* $C0C7 */
	NULL,			/* $C0C8 */
	NULL,			/* $C0C9 */
	NULL,			/* $C0CA */
	NULL,			/* $C0CB */
	NULL,			/* $C0CC */
	NULL,			/* $C0CD */
	NULL,			/* $C0CE */
	NULL,			/* $C0CF */
	NULL,			/* $C0D0 */
	NULL,			/* $C0D1 */
	NULL,			/* $C0D2 */
	NULL,			/* $C0D3 */
	NULL,			/* $C0D4 */
	NULL,			/* $C0D5 */
	NULL,			/* $C0D6 */
	NULL,			/* $C0D7 */
	NULL,			/* $C0D8 */
	NULL,			/* $C0D9 */
	NULL,			/* $C0DA */
	NULL,			/* $C0DB */
	NULL,			/* $C0DC */
	NULL,			/* $C0DD */
	NULL,			/* $C0DE */
	NULL,			/* $C0DF */
	NULL,			/* $C0E0 */
	NULL,			/* $C0E1 */
	NULL,			/* $C0E2 */
	NULL,			/* $C0E3 */
	NULL,			/* $C0E4 */
	NULL,			/* $C0E5 */
	NULL,			/* $C0E6 */
	NULL,			/* $C0E7 */
	NULL,			/* $C0E8 */
	NULL,			/* $C0E9 */
	NULL,			/* $C0EA */
	NULL,			/* $C0EB */
	diskfake1,		/* $C0EC */
	NULL,			/* $C0ED */
	diskfake2,		/* $C0EE */
	NULL,			/* $C0EF */
	NULL,			/* $C0F0 */
	NULL,			/* $C0F1 */
	NULL,			/* $C0F2 */
	NULL,			/* $C0F3 */
	NULL,			/* $C0F4 */
	NULL,			/* $C0F5 */
	NULL,			/* $C0F6 */
	NULL,			/* $C0F7 */
	NULL,			/* $C0F8 */
	NULL,			/* $C0F9 */
	NULL,			/* $C0FA */
	NULL,			/* $C0FB */
	NULL,			/* $C0FC */
	NULL,			/* $C0FD */
	NULL,			/* $C0FE */
	NULL /* $C0FF */
};

byte (*mem_io_write_dispatch[256])(byte) = {
	MEM_clear80store,	/* $C000 */
	MEM_set80store,		/* $C001 */
	MEM_clearAuxRd,		/* $C002 */
	MEM_setAuxRd,		/* $C003 */
	MEM_clearAuxWrt,	/* $C004 */
	MEM_setAuxWrt,		/* $C005 */
	NULL,			/* $C006 */
	NULL,			/* $C007 */
	MEM_clearAltZP,		/* $C008 */
	MEM_setAltZP,		/* $C009 */
	NULL,			/* $C00A */
	NULL,			/* $C00B */
	VID_clear80col,		/* $C00C */
	VID_set80col,		/* $C00D */
	VID_clearAltCh,		/* $C00E */
	VID_setAltCh,		/* $C00F */
	ADB_clearKey,		/* $C010 */
	NULL,			/* $C011 */
	NULL,			/* $C012 */
	NULL,			/* $C013 */
	NULL,			/* $C014 */
	NULL,			/* $C015 */
	NULL,			/* $C016 */
	NULL,			/* $C017 */
	NULL,			/* $C018 */
	NULL,			/* $C019 */
	NULL,			/* $C01A */
	NULL,			/* $C01B */
	NULL,			/* $C01C */
	NULL,			/* $C01D */
	NULL,			/* $C01E */
	NULL,			/* $C01F */
	NULL,			/* $C020 */
	NULL,			/* $C021 */
	VID_setColorReg,	/* $C022 */
	NULL,			/* $C023 */
	NULL,			/* $C024 */
	NULL,			/* $C025 */
	ADB_setCommand,		/* $C026 */
	ADB_setStatus,		/* $C027 */
	NULL,			/* $C028 */
	VID_setNewVideo,	/* $C029 */
	NULL,			/* $C02A */
	NULL,			/* $C02B */
	NULL,			/* $C02C */
	MEM_setSlotReg,		/* $C02D */
	NULL,			/* $C02E */
	NULL,			/* $C02F */
	NULL,			/* $C030 */
	NULL,			/* $C031 */
	NULL,			/* $C032 */
	CLK_setData,		/* $C033 */
	CLK_setControl,		/* $C034 */
	MEM_setShadowReg,	/* $C035 */
	NULL,			/* $C036 */
	NULL,			/* $C037 */
	NULL,			/* $C038 */
	NULL,			/* $C039 */
	NULL,			/* $C03A */
	NULL,			/* $C03B */
	SND_writeSoundCtl,	/* $C03C */
	SND_writeSoundData,	/* $C03D */
	SND_writeSoundAddrL,	/* $C03E */
	SND_writeSoundAddrH,	/* $C03F */
	NULL,			/* $C040 */
	MEM_setIntEnReg,	/* $C041 */
	NULL,			/* $C042 */
	NULL,			/* $C043 */
	NULL,			/* $C044 */
	NULL,			/* $C045 */
	NULL,			/* $C046 */
	MEM_setVBLClear,	/* $C047 */
	NULL,			/* $C048 */
	NULL,			/* $C049 */
	NULL,			/* $C04A */
	NULL,			/* $C04B */
	NULL,			/* $C04C */
	NULL,			/* $C04D */
	NULL,			/* $C04E */
	NULL,			/* $C04F */
	VID_clearText,		/* $C050 */
	VID_setText,		/* $C051 */
	VID_clearMixed,		/* $C052 */
	VID_setMixed,		/* $C053 */
	VID_clearPage2,		/* $C054 */
	VID_setPage2,		/* $C055 */
	VID_clearHires,		/* $C056 */
	VID_setHires,		/* $C057 */
	NULL,			/* $C058 */
	NULL,			/* $C059 */
	NULL,			/* $C05A */
	NULL,			/* $C05B */
	NULL,			/* $C05C */
	NULL,			/* $C05D */
	VID_setDblRes,		/* $C05E */
	VID_clearDblRes,	/* $C05F */
	NULL,			/* $C060 */
	NULL,			/* $C061 */
	NULL,			/* $C062 */
	NULL,			/* $C063 */
	NULL,			/* $C064 */
	NULL,			/* $C065 */
	NULL,			/* $C066 */
	NULL,			/* $C067 */
	MEM_setStateReg,	/* $C068 */
	NULL,			/* $C069 */
	NULL,			/* $C06A */
	NULL,			/* $C06B */
	NULL,			/* $C06C */
	NULL,			/* $C06D */
	NULL,			/* $C06E */
	NULL,			/* $C06F */
	NULL,			/* $C070 */
	NULL,			/* $C071 */
	NULL,			/* $C072 */
	NULL,			/* $C073 */
	NULL,			/* $C074 */
	NULL,			/* $C075 */
	NULL,			/* $C076 */
	NULL,			/* $C077 */
	NULL,			/* $C078 */
	NULL,			/* $C079 */
	NULL,			/* $C07A */
	NULL,			/* $C07B */
	NULL,			/* $C07C */
	NULL,			/* $C07D */
	NULL,			/* $C07E */
	NULL,			/* $C07F */
	MEM_setLCx80,		/* $C080 */
	MEM_setLCx81,		/* $C081 */
	MEM_setLCx82,		/* $C082 */
	MEM_setLCx83,		/* $C083 */
	NULL,			/* $C084 */
	NULL,			/* $C085 */
	NULL,			/* $C086 */
	NULL,			/* $C087 */
	MEM_setLCx88,		/* $C088 */
	MEM_setLCx89,		/* $C089 */
	MEM_setLCx8A,		/* $C08A */
	MEM_setLCx8B,		/* $C08B */
	NULL,			/* $C08C */
	NULL,			/* $C08D */
	NULL,			/* $C08E */
	NULL,			/* $C08F */
	NULL,			/* $C090 */
	NULL,			/* $C091 */
	NULL,			/* $C092 */
	NULL,			/* $C093 */
	NULL,			/* $C094 */
	NULL,			/* $C095 */
	NULL,			/* $C096 */
	NULL,			/* $C097 */
	NULL,			/* $C098 */
	NULL,			/* $C099 */
	NULL,			/* $C09A */
	NULL,			/* $C09B */
	NULL,			/* $C09C */
	NULL,			/* $C09D */
	NULL,			/* $C09E */
	NULL,			/* $C09F */
	NULL,			/* $C0A0 */
	NULL,			/* $C0A1 */
	NULL,			/* $C0A2 */
	NULL,			/* $C0A3 */
	NULL,			/* $C0A4 */
	NULL,			/* $C0A5 */
	NULL,			/* $C0A6 */
	NULL,			/* $C0A7 */
	NULL,			/* $C0A8 */
	NULL,			/* $C0A9 */
	NULL,			/* $C0AA */
	NULL,			/* $C0AB */
	NULL,			/* $C0AC */
	NULL,			/* $C0AD */
	NULL,			/* $C0AE */
	NULL,			/* $C0AF */
	NULL,			/* $C0B0 */
	NULL,			/* $C0B1 */
	NULL,			/* $C0B2 */
	NULL,			/* $C0B3 */
	NULL,			/* $C0B4 */
	NULL,			/* $C0B5 */
	NULL,			/* $C0B6 */
	NULL,			/* $C0B7 */
	NULL,			/* $C0B8 */
	NULL,			/* $C0B9 */
	NULL,			/* $C0BA */
	NULL,			/* $C0BB */
	NULL,			/* $C0BC */
	NULL,			/* $C0BD */
	NULL,			/* $C0BE */
	NULL,			/* $C0BF */
	NULL,			/* $C0C0 */
	NULL,			/* $C0C1 */
	NULL,			/* $C0C2 */
	NULL,			/* $C0C3 */
	NULL,			/* $C0C4 */
	NULL,			/* $C0C5 */
	NULL,			/* $C0C6 */
	NULL,			/* $C0C7 */
	NULL,			/* $C0C8 */
	NULL,			/* $C0C9 */
	NULL,			/* $C0CA */
	NULL,			/* $C0CB */
	NULL,			/* $C0CC */
	NULL,			/* $C0CD */
	NULL,			/* $C0CE */
	NULL,			/* $C0CF */
	NULL,			/* $C0D0 */
	NULL,			/* $C0D1 */
	NULL,			/* $C0D2 */
	NULL,			/* $C0D3 */
	NULL,			/* $C0D4 */
	NULL,			/* $C0D5 */
	NULL,			/* $C0D6 */
	NULL,			/* $C0D7 */
	NULL,			/* $C0D8 */
	NULL,			/* $C0D9 */
	NULL,			/* $C0DA */
	NULL,			/* $C0DB */
	NULL,			/* $C0DC */
	NULL,			/* $C0DD */
	NULL,			/* $C0DE */
	NULL,			/* $C0DF */
	NULL,			/* $C0E0 */
	NULL,			/* $C0E1 */
	NULL,			/* $C0E2 */
	NULL,			/* $C0E3 */
	NULL,			/* $C0E4 */
	NULL,			/* $C0E5 */
	NULL,			/* $C0E6 */
	NULL,			/* $C0E7 */
	NULL,			/* $C0E8 */
	NULL,			/* $C0E9 */
	NULL,			/* $C0EA */
	NULL,			/* $C0EB */
	NULL,			/* $C0EC */
	NULL,			/* $C0ED */
	NULL,			/* $C0EE */
	diskfake3,		/* $C0EF */
	NULL,			/* $C0F0 */
	NULL,			/* $C0F1 */
	NULL,			/* $C0F2 */
	NULL,			/* $C0F3 */
	NULL,			/* $C0F4 */
	NULL,			/* $C0F5 */
	NULL,			/* $C0F6 */
	NULL,			/* $C0F7 */
	NULL,			/* $C0F8 */
	NULL,			/* $C0F9 */
	NULL,			/* $C0FA */
	NULL,			/* $C0FB */
	NULL,			/* $C0FC */
	NULL,			/* $C0FD */
	NULL,			/* $C0FE */
	NULL			/* $C0FF */
};

/* Default read fault handler; just returns 0. */

byte MEM_defaultReadFaultHandler(address addr)
{
	return 0;
}

/* Default write fault handler; does nothing. */

void MEM_defaultWriteFaultHandler(address addr,byte val)
{
	register int	bank;

	bank = (addr >> 16) & 0xFF;
	addr &= 0xFFFF;
	*(memory[bank] + addr) = val;
	if (bank < 0x02) *(memory[bank | 0xE0] + addr) = val;
}

/* Read fault handler for reads to page $C0 I/O space. */

byte MEM_ioReadFaultHandler(address addr)
{
	register byte	(*func)(byte);

	func = mem_io_read_dispatch[addr & 0xFF];
	if (!func) return 0;
	return (*func)(0);
}

/* Write fault handler for writes to page $C0 I/O space. */

void MEM_ioWriteFaultHandler(address addr, byte val)
{
	register byte	(*func)(byte);

	func = mem_io_write_dispatch[addr & 0xFF];
	if (func) (*func)(val);
}
